/////////////////////// using GPIO /////////////////////////////////
#define SDA1_BIT    GPIO_Pin_11
#define SCL1_BIT    GPIO_Pin_10
#define I2C_PORT    GPIOB

unsigned volatile int ucNack_C;

#define SDA1_Ready()     (I2C_PORT->IDR & SDA1_BIT)

#define nop()   asm(" nop ")
void nops(u8 num){
u8 i;
    for(i=0;i<num;i++) nop();
}
/*******************************************************************************
* Function Name : void GPIO_I2C_Init(void)
* Description   : I2C   ʰ I/O Ͽ  ۼ   
* Parameters    : None
* Return        : None
*******************************************************************************/
void GPIO_I2C_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    // SDA, SCK Configuration
    GPIO_InitStructure.GPIO_Pin = SCL1_BIT | SDA1_BIT;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(I2C_PORT, &GPIO_InitStructure); 

    GPIO_SetBits(I2C_PORT, SCL1_BIT);
	GPIO_SetBits(I2C_PORT, SDA1_BIT);
}
/*******************************************************************************
* Function Name : void GPIO_I2C_Start(void)
* Description   : Data ۼ 
* Parameters    : None
* Return        : None
*******************************************************************************/
void GPIO_I2C_Start(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    // SDA, SCK Configuration
    GPIO_InitStructure.GPIO_Pin = SCL1_BIT | SDA1_BIT;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;   // Output 
    GPIO_Init(I2C_PORT, &GPIO_InitStructure); 
    
    GPIO_SetBits(I2C_PORT, SCL1_BIT);
	GPIO_SetBits(I2C_PORT, SDA1_BIT);
    delay_us(1);

    GPIO_ResetBits(I2C_PORT, SDA1_BIT);
    delay_us(4);
    GPIO_ResetBits(I2C_PORT, SCL1_BIT);
}
/*******************************************************************************
* Function Name : void GPIO_I2C_Stop(void)
* Description   : Data ۼ 
* Parameters    : None
* Return        : None
*******************************************************************************/
void GPIO_I2C_Stop(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    // SDA Configuration
    GPIO_InitStructure.GPIO_Pin = SDA1_BIT;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;   // Output 
    GPIO_Init(I2C_PORT, &GPIO_InitStructure);    
    
    GPIO_ResetBits(I2C_PORT, SDA1_BIT);
	GPIO_ResetBits(I2C_PORT, SCL1_BIT);
    delay_us(4);
    
    GPIO_SetBits(I2C_PORT, SCL1_BIT);
    delay_us(3);
    GPIO_SetBits(I2C_PORT, SDA1_BIT);
    
    // SDA Configuration
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;   // Input 
    GPIO_Init(I2C_PORT, &GPIO_InitStructure);      
}
/*******************************************************************************
* Function Name : void FIFO_Clock(void)
* Description   : Data ۼ Ŭ ߻
* Parameters    : None
* Return        : None
******************************************************************************/
void GPIO_I2C_Clock(void)
{
    GPIO_SetBits(I2C_PORT, SCL1_BIT);    
    delay_us(2);
    GPIO_ResetBits(I2C_PORT, SCL1_BIT);
}
/*******************************************************************************
* Function Name : u8 GPIO_I2C_Ackn(void)
* Description   : ackn flag  üũ
* Parameters    : None
* Return        : ackn flag data~~
*******************************************************************************/
u8 GPIO_I2C_Ackn(void)
{
    u8 ackn = 0;  // Temp RAM for Ackn flag
    
    GPIO_InitTypeDef GPIO_InitStructure;

    // SDA Configuration
    GPIO_InitStructure.GPIO_Pin = SDA1_BIT;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;  
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;  // Input 
    GPIO_Init(I2C_PORT, &GPIO_InitStructure);     
    
    GPIO_ResetBits(I2C_PORT, SCL1_BIT); 
    GPIO_SetBits(I2C_PORT, SCL1_BIT);
    nops(1);
    
    if(SDA1_Ready())		ackn = 1;
	else 				ackn = 0;
	
    nops(1);

    GPIO_ResetBits(I2C_PORT, SCL1_BIT);
	
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;  // Output 
    GPIO_Init(I2C_PORT, &GPIO_InitStructure); 

    return ackn;
}
/*******************************************************************************
* Function Name : void Write_I2C_Byte(u8 byte)
* Description   : Write_I2C Data Write
* Parameters    : None
* Return        :   8Ʈ
******************************************************************************/
void Write_I2C_Byte(u8 byte)
{
    u8 i;
 
    GPIO_InitTypeDef GPIO_InitStructure;

    // SDA Configuration
    GPIO_InitStructure.GPIO_Pin = SDA1_BIT;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;  // Output 
    GPIO_Init(I2C_PORT, &GPIO_InitStructure);    

    GPIO_ResetBits(I2C_PORT, SCL1_BIT); 
    
    for(i=0; i<8; i++)
    {
        if((byte & 0x80) == 0x80)   GPIO_SetBits(I2C_PORT, SDA1_BIT);
        else                        GPIO_ResetBits(I2C_PORT, SDA1_BIT);
        nops(1);
        
        GPIO_SetBits(I2C_PORT, SCL1_BIT);
        byte = byte << 1;
        nops(1);
        if(i == 0)  nops(2);
        GPIO_ResetBits(I2C_PORT, SCL1_BIT);
    }
    
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;  // Input 
    GPIO_Init(I2C_PORT, &GPIO_InitStructure);   
    
    nops(45);
    
    if(GPIO_I2C_Ackn() == 1)  	ucNack_C = 1;
}
/*******************************************************************************
* Function Name : u8 Read_I2C_Byte(u8 ack)
* Description   : FIFO Data Read
* Parameters    : ack ÷  
* Return        : FIFO Read data return 8 bit
*******************************************************************************/
u8 Read_I2C_Byte(u8 ack)
{
    u8 i, buff = 0;
    
    GPIO_InitTypeDef GPIO_InitStructure;

    // SDA Configuration
    GPIO_InitStructure.GPIO_Pin = SDA1_BIT;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;  // Input 
    GPIO_Init(I2C_PORT, &GPIO_InitStructure);   
    nop();
    
    for(i=0; i<8; i++)
    {
        buff <<= 1;
        
         GPIO_SetBits(I2C_PORT, SCL1_BIT);
         nops(3);
         
         if(SDA1_Ready())  buff |= 0x01;
         
         GPIO_ResetBits(I2C_PORT, SCL1_BIT);
         nops(3);
    }
    
    nops(80);
    
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;  // Output 
    GPIO_Init(I2C_PORT, &GPIO_InitStructure);       
    
    if(ack == 0)
    {
        GPIO_ResetBits(I2C_PORT, SDA1_BIT);
        GPIO_I2C_Clock();
    }
    else
    {
        GPIO_SetBits(I2C_PORT, SDA1_BIT);
        GPIO_I2C_Clock();
        nops(4);
        GPIO_ResetBits(I2C_PORT, SDA1_BIT);
    }
    
    return buff;
}
/*******************************************************************************
* Function Name : void TouchWrite(u8 add, u8 data)
* Description   : Touch  Write
* Parameters    : FIFO ּҿ Է  
* Return        : None
*******************************************************************************/
void TouchWrite(u8 add, u8 data)
{
    ucNack_C = 0;

    do{
      GPIO_I2C_Start();
      
      Write_I2C_Byte(add);
      delay_us(100);
      
      Write_I2C_Byte(data);
      delay_us(70);
      
      GPIO_I2C_Stop();	 
    } while(ucNack_C == 1);
}

/*******************************************************************************
* Function Name : u16 TouchRead(u8 add)
* Description   : Touch  Read
* Parameters    :   ּ
* Return        :  ּ  
*******************************************************************************/
u16 TouchRead(u8 add)
{
    u16 ret = 0;
    ucNack_C = 0;
    
    do{
      GPIO_I2C_Start();
      
      Write_I2C_Byte(add);
      delay_us(100);
      
      ret = Read_I2C_Byte(0)<<4;
      delay_us(10);
      
	  ret |= Read_I2C_Byte(1)>>4;
      delay_us(70);  
	  
	  GPIO_I2C_Stop();
   } while(ucNack_C == 1);
    
    return ret;
}